@@ -21,8 +21,6 @@
 #include <net/netfilter/nf_conntrack.h>
 #include <net/netfilter/nf_nat_core.h>
 
-#define MAX_QUEUED_PACKETS 1024
-
 static int init(struct net_device *dev)
 {
 	dev->tstats = netdev_alloc_pcpu_stats(struct pcpu_sw_netstats);
@@ -111,28 +109,24 @@ static netdev_tx_t xmit(struct sk_buff *skb, struct net_device *dev)
 
 	peer = routing_table_lookup_dst(&wg->peer_routing_table, skb);
 	if (unlikely(!peer)) {
-#if defined(CONFIG_DYNAMIC_DEBUG) || defined(DEBUG)
-		struct sockaddr_storage addr;
-		socket_addr_from_skb(&addr, skb);
-		net_dbg_ratelimited("No peer is configured for %pISc\n", &addr);
-#endif
+		net_dbg_skb_ratelimited("No peer is configured for %pISc\n", skb);
 		skb_unsendable(skb, dev);
 		return -ENOKEY;
 	}
 
 	read_lock_bh(&peer->endpoint_lock);
-	ret = peer->endpoint_addr.ss_family != AF_INET && peer->endpoint_addr.ss_family != AF_INET6;
+	ret = peer->endpoint.addr_storage.ss_family != AF_INET && peer->endpoint.addr_storage.ss_family != AF_INET6;
 	read_unlock_bh(&peer->endpoint_lock);
 	if (unlikely(ret)) {
-		net_dbg_ratelimited("No valid endpoint has been configured or discovered for device\n");
-		peer_put(peer);
+		net_dbg_ratelimited("No valid endpoint has been configured or discovered for peer %Lu\n", peer->internal_id);
 		skb_unsendable(skb, dev);
+		peer_put(peer);
 		return -EHOSTUNREACH;
 	}
 
 	/* If the queue is getting too big, we start removing the oldest packets until it's small again.
 	 * We do this before adding the new packet, so we don't remove GSO segments that are in excess. */
-	while (skb_queue_len(&peer->tx_packet_queue) > MAX_QUEUED_PACKETS)
+	while (skb_queue_len(&peer->tx_packet_queue) > MAX_QUEUED_OUTGOING_PACKETS)
 		dev_kfree_skb(skb_dequeue(&peer->tx_packet_queue));
 
 	if (!skb_is_gso(skb))
@@ -221,7 +215,9 @@ static void destruct(struct net_device *dev)
 	free_netdev(dev);
 }
 
-#define WG_FEATURES (NETIF_F_HW_CSUM | NETIF_F_RXCSUM | NETIF_F_SG | NETIF_F_GSO | NETIF_F_GSO_SOFTWARE | NETIF_F_HIGHDMA)
+enum {
+	WG_NETDEV_FEATURES = NETIF_F_HW_CSUM | NETIF_F_RXCSUM | NETIF_F_SG | NETIF_F_GSO | NETIF_F_GSO_SOFTWARE | NETIF_F_HIGHDMA
+};
 
 static void setup(struct net_device *dev)
 {
@@ -233,17 +229,17 @@ static void setup(struct net_device *dev)
 	dev->addr_len = 0;
 	dev->needed_headroom = DATA_PACKET_HEAD_ROOM;
 	dev->needed_tailroom = noise_encrypted_len(MESSAGE_PADDING_MULTIPLE);
-	dev->type = ARPHRD_VOID; /* Virtually the same as ARPHRD_NONE, except doesn't get IP6 auto config. */
-	dev->flags = IFF_POINTOPOINT | IFF_NOARP | IFF_MULTICAST;
+	dev->type = ARPHRD_NONE; /* Virtually the same as ARPHRD_NONE, except doesn't get IP6 auto config. */
+	dev->flags = IFF_POINTOPOINT | IFF_NOARP;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 3, 0)
 	dev->flags |= IFF_NO_QUEUE;
 #else
 	dev->tx_queue_len = 0;
 #endif
 	dev->features |= NETIF_F_LLTX;
-	dev->features |= WG_FEATURES;
-	dev->hw_features |= WG_FEATURES;
-	dev->hw_enc_features |= WG_FEATURES;
+	dev->features |= WG_NETDEV_FEATURES;
+	dev->hw_features |= WG_NETDEV_FEATURES;
+	dev->hw_enc_features |= WG_NETDEV_FEATURES;
 	dev->mtu = ETH_DATA_LEN - MESSAGE_MINIMUM_LENGTH - sizeof(struct udphdr) - max(sizeof(struct ipv6hdr), sizeof(struct iphdr));
 
 	/* We need to keep the dst around in case of icmp replies. */
