@@ -25,20 +25,14 @@ static inline void rx_stats(struct wireguard_peer *peer, size_t len)
 
 static inline void update_latest_addr(struct wireguard_peer *peer, struct sk_buff *skb)
 {
-	struct sockaddr_storage addr = { 0 };
-	if (!socket_addr_from_skb(&addr, skb))
-		socket_set_peer_addr(peer, &addr);
+	struct endpoint endpoint;
+	if (!socket_endpoint_from_skb(&endpoint, skb))
+		socket_set_peer_endpoint(peer, &endpoint);
 }
 
 static inline int skb_data_offset(struct sk_buff *skb, size_t *data_offset, size_t *data_len)
 {
 	struct udphdr *udp;
-#if defined(CONFIG_DYNAMIC_DEBUG) || defined(DEBUG)
-	struct sockaddr_storage addr = { 0 };
-	socket_addr_from_skb(&addr, skb);
-#else
-	static const u8 addr;
-#endif
 
 	if (unlikely(skb->len < sizeof(struct iphdr)))
 		return -EINVAL;
@@ -50,26 +44,26 @@ static inline int skb_data_offset(struct sk_buff *skb, size_t *data_offset, size
 	udp = udp_hdr(skb);
 	*data_offset = (u8 *)udp - skb->data;
 	if (unlikely(*data_offset > U16_MAX)) {
-		net_dbg_ratelimited("Packet has offset at impossible location from %pISpfsc\n", &addr);
+		net_dbg_skb_ratelimited("Packet has offset at impossible location from %pISpfsc\n", skb);
 		return -EINVAL;
 	}
 	if (unlikely(*data_offset + sizeof(struct udphdr) > skb->len)) {
-		net_dbg_ratelimited("Packet isn't big enough to have UDP fields from %pISpfsc\n", &addr);
+		net_dbg_skb_ratelimited("Packet isn't big enough to have UDP fields from %pISpfsc\n", skb);
 		return -EINVAL;
 	}
 	*data_len = ntohs(udp->len);
 	if (unlikely(*data_len < sizeof(struct udphdr))) {
-		net_dbg_ratelimited("UDP packet is reporting too small of a size from %pISpfsc\n", &addr);
+		net_dbg_skb_ratelimited("UDP packet is reporting too small of a size from %pISpfsc\n", skb);
 		return -EINVAL;
 	}
 	if (unlikely(*data_len > skb->len - *data_offset)) {
-		net_dbg_ratelimited("UDP packet is lying about its size from %pISpfsc\n", &addr);
+		net_dbg_skb_ratelimited("UDP packet is lying about its size from %pISpfsc\n", skb);
 		return -EINVAL;
 	}
 	*data_len -= sizeof(struct udphdr);
 	*data_offset = (u8 *)udp + sizeof(struct udphdr) - skb->data;
 	if (!pskb_may_pull(skb, *data_offset + sizeof(struct message_header))) {
-		net_dbg_ratelimited("Could not pull header into data section from %pISpfsc\n", &addr);
+		net_dbg_skb_ratelimited("Could not pull header into data section from %pISpfsc\n", skb);
 		return -EINVAL;
 	}
 
@@ -84,29 +78,22 @@ static void receive_handshake_packet(struct wireguard_device *wg, void *data, si
 	enum cookie_mac_state mac_state;
 	bool packet_needs_cookie;
 
-#if defined(CONFIG_DYNAMIC_DEBUG) || defined(DEBUG)
-	struct sockaddr_storage addr = { 0 };
-	socket_addr_from_skb(&addr, skb);
-#else
-	static const u8 addr;
-#endif
-
 	message_type = message_determine_type(data, len);
 
 	if (message_type == MESSAGE_HANDSHAKE_COOKIE) {
-		net_dbg_ratelimited("Receiving cookie response from %pISpfsc\n", &addr);
+		net_dbg_skb_ratelimited("Receiving cookie response from %pISpfsc\n", skb);
 		cookie_message_consume(data, wg);
 		return;
 	}
 
-	under_load = skb_queue_len(&wg->incoming_handshakes) >= MAX_QUEUED_HANDSHAKES / 2;
+	under_load = skb_queue_len(&wg->incoming_handshakes) >= MAX_QUEUED_INCOMING_HANDSHAKES / 2;
 	mac_state = cookie_validate_packet(&wg->cookie_checker, skb, data, len, under_load);
 	if ((under_load && mac_state == VALID_MAC_WITH_COOKIE) || (!under_load && mac_state == VALID_MAC_BUT_NO_COOKIE))
 		packet_needs_cookie = false;
 	else if (under_load && mac_state == VALID_MAC_BUT_NO_COOKIE)
 		packet_needs_cookie = true;
 	else {
-		net_dbg_ratelimited("Invalid MAC of handshake, dropping packet from %pISpfsc\n", &addr);
+		net_dbg_skb_ratelimited("Invalid MAC of handshake, dropping packet from %pISpfsc\n", skb);
 		return;
 	}
 
@@ -119,11 +106,11 @@ static void receive_handshake_packet(struct wireguard_device *wg, void *data, si
 		}
 		peer = noise_handshake_consume_initiation(data, wg);
 		if (unlikely(!peer)) {
-			net_dbg_ratelimited("Invalid handshake initiation from %pISpfsc\n", &addr);
+			net_dbg_skb_ratelimited("Invalid handshake initiation from %pISpfsc\n", skb);
 			return;
 		}
-		net_dbg_ratelimited("Receiving handshake initiation from peer %Lu (%pISpfsc)\n", peer->internal_id, &addr);
 		update_latest_addr(peer, skb);
+		net_dbg_ratelimited("Receiving handshake initiation from peer %Lu (%pISpfsc)\n", peer->internal_id, &peer->endpoint.addr_storage);
 		packet_send_handshake_response(peer);
 		break;
 	case MESSAGE_HANDSHAKE_RESPONSE:
@@ -134,10 +121,11 @@ static void receive_handshake_packet(struct wireguard_device *wg, void *data, si
 		}
 		peer = noise_handshake_consume_response(data, wg);
 		if (unlikely(!peer)) {
-			net_dbg_ratelimited("Invalid handshake response from %pISpfsc\n", &addr);
+			net_dbg_skb_ratelimited("Invalid handshake response from %pISpfsc\n", skb);
 			return;
 		}
-		net_dbg_ratelimited("Receiving handshake response from peer %Lu (%pISpfsc)\n", peer->internal_id, &addr);
+		update_latest_addr(peer, skb);
+		net_dbg_ratelimited("Receiving handshake response from peer %Lu (%pISpfsc)\n", peer->internal_id, &peer->endpoint.addr_storage);
 		if (noise_handshake_begin_session(&peer->handshake, &peer->keypairs, true)) {
 			timers_ephemeral_key_created(peer);
 			timers_handshake_complete(peer);
@@ -150,8 +138,7 @@ static void receive_handshake_packet(struct wireguard_device *wg, void *data, si
 		}
 		break;
 	default:
-		net_err_ratelimited("Somehow a wrong type of packet wound up in the handshake queue from %pISpfsc!\n", &addr);
-		BUG();
+		WARN(1, "Somehow a wrong type of packet wound up in the handshake queue!\n");
 		return;
 	}
 
@@ -160,7 +147,6 @@ static void receive_handshake_packet(struct wireguard_device *wg, void *data, si
 	rx_stats(peer, len);
 	timers_any_authenticated_packet_received(peer);
 	timers_any_authenticated_packet_traversal(peer);
-	update_latest_addr(peer, skb);
 	peer_put(peer);
 }
 
@@ -175,7 +161,7 @@ void packet_process_queued_handshake_packets(struct work_struct *work)
 		if (!skb_data_offset(skb, &offset, &len))
 			receive_handshake_packet(wg, skb->data + offset, len, skb);
 		dev_kfree_skb(skb);
-		if (++num_processed == MAX_BURST_HANDSHAKES) {
+		if (++num_processed == MAX_BURST_INCOMING_HANDSHAKES) {
 			queue_work(wg->workqueue, &wg->incoming_handshakes_work);
 			return;
 		}
@@ -207,17 +193,19 @@ struct packet_cb {
 };
 #define PACKET_CB(skb) ((struct packet_cb *)skb->cb)
 
-static void receive_data_packet(struct sk_buff *skb, struct wireguard_peer *peer, struct sockaddr_storage *addr, bool used_new_key, int err)
+static void receive_data_packet(struct sk_buff *skb, struct wireguard_peer *peer, struct endpoint *endpoint, bool used_new_key, int err)
 {
 	struct net_device *dev;
 	struct wireguard_peer *routed_peer;
 	struct wireguard_device *wg;
 
-	if (unlikely(err < 0 || !peer || !addr)) {
+	if (unlikely(err < 0 || !peer || !endpoint)) {
 		dev_kfree_skb(skb);
 		return;
 	}
 
+	socket_set_peer_endpoint(peer, endpoint);
+
 	wg = peer->device;
 	dev = netdev_pub(wg);
 
@@ -230,21 +218,21 @@ static void receive_data_packet(struct sk_buff *skb, struct wireguard_peer *peer
 
 	/* A packet with length 0 is a keepalive packet */
 	if (unlikely(!skb->len)) {
-		net_dbg_ratelimited("Receiving keepalive packet from peer %Lu (%pISpfsc)\n", peer->internal_id, addr);
+		net_dbg_ratelimited("Receiving keepalive packet from peer %Lu (%pISpfsc)\n", peer->internal_id, &peer->endpoint.addr_storage);
 		goto packet_processed;
 	}
 
 	if (unlikely(skb->len < sizeof(struct iphdr))) {
 		++dev->stats.rx_errors;
 		++dev->stats.rx_length_errors;
-		net_dbg_ratelimited("Packet missing ip header from peer %Lu (%pISpfsc)\n", peer->internal_id, addr);
+		net_dbg_ratelimited("Packet missing ip header from peer %Lu (%pISpfsc)\n", peer->internal_id, &peer->endpoint.addr_storage);
 		goto packet_processed;
 	}
 
 	if (!pskb_may_pull(skb, 1 /* For checking the ip version below */)) {
 		++dev->stats.rx_errors;
 		++dev->stats.rx_length_errors;
-		net_dbg_ratelimited("Packet missing IP version from peer %Lu (%pISpfsc)\n", peer->internal_id, addr);
+		net_dbg_ratelimited("Packet missing IP version from peer %Lu (%pISpfsc)\n", peer->internal_id, &peer->endpoint.addr_storage);
 		goto packet_processed;
 	}
 
@@ -258,7 +246,7 @@ static void receive_data_packet(struct sk_buff *skb, struct wireguard_peer *peer
 		if (unlikely(skb->len < sizeof(struct ipv6hdr))) {
 			++dev->stats.rx_errors;
 			++dev->stats.rx_length_errors;
-			net_dbg_ratelimited("Packet missing ipv6 header from peer %Lu (%pISpfsc)\n", peer->internal_id, addr);
+			net_dbg_ratelimited("Packet missing ipv6 header from peer %Lu (%pISpfsc)\n", peer->internal_id, &peer->endpoint.addr_storage);
 			goto packet_processed;
 		}
 		skb->protocol = htons(ETH_P_IPV6);
@@ -267,7 +255,7 @@ static void receive_data_packet(struct sk_buff *skb, struct wireguard_peer *peer
 	} else {
 		++dev->stats.rx_errors;
 		++dev->stats.rx_length_errors;
-		net_dbg_ratelimited("Packet neither ipv4 nor ipv6 from peer %Lu (%pISpfsc)\n", peer->internal_id, addr);
+		net_dbg_ratelimited("Packet neither ipv4 nor ipv6 from peer %Lu (%pISpfsc)\n", peer->internal_id, &peer->endpoint.addr_storage);
 		goto packet_processed;
 	}
 
@@ -277,13 +265,9 @@ static void receive_data_packet(struct sk_buff *skb, struct wireguard_peer *peer
 	peer_put(routed_peer); /* We don't need the extra reference. */
 
 	if (unlikely(routed_peer != peer)) {
-#if defined(CONFIG_DYNAMIC_DEBUG) || defined(DEBUG)
-		struct sockaddr_storage unencrypted_addr = { 0 };
-		socket_addr_from_skb(&unencrypted_addr, skb);
-		net_dbg_ratelimited("Packet has unallowed src IP (%pISc) from peer %Lu (%pISpfsc)\n", &unencrypted_addr, peer->internal_id, addr);
-#endif
 		++dev->stats.rx_errors;
 		++dev->stats.rx_frame_errors;
+		net_dbg_skb_ratelimited("Packet has unallowed src IP (%pISc) from peer %Lu (%pISpfsc)\n", skb, peer->internal_id, &peer->endpoint.addr_storage);
 		goto packet_processed;
 	}
 
@@ -292,7 +276,7 @@ static void receive_data_packet(struct sk_buff *skb, struct wireguard_peer *peer
 		rx_stats(peer, skb->len);
 	else {
 		++dev->stats.rx_dropped;
-		net_dbg_ratelimited("Failed to give packet to userspace from peer %Lu (%pISpfsc)\n", peer->internal_id, addr);
+		net_dbg_ratelimited("Failed to give packet to userspace from peer %Lu (%pISpfsc)\n", peer->internal_id, &peer->endpoint.addr_storage);
 	}
 	goto continue_processing;
 
@@ -301,19 +285,12 @@ packet_processed:
 continue_processing:
 	timers_any_authenticated_packet_received(peer);
 	timers_any_authenticated_packet_traversal(peer);
-	socket_set_peer_addr(peer, addr);
 	peer_put(peer);
 }
 
 void packet_receive(struct wireguard_device *wg, struct sk_buff *skb)
 {
 	size_t len, offset;
-#if defined(CONFIG_DYNAMIC_DEBUG) || defined(DEBUG)
-	struct sockaddr_storage addr = { 0 };
-	socket_addr_from_skb(&addr, skb);
-#else
-	static const u8 addr;
-#endif
 
 	if (unlikely(skb_data_offset(skb, &offset, &len) < 0))
 		goto err;
@@ -321,12 +298,12 @@ void packet_receive(struct wireguard_device *wg, struct sk_buff *skb)
 	case MESSAGE_HANDSHAKE_INITIATION:
 	case MESSAGE_HANDSHAKE_RESPONSE:
 	case MESSAGE_HANDSHAKE_COOKIE:
-		if (skb_queue_len(&wg->incoming_handshakes) > MAX_QUEUED_HANDSHAKES) {
-			net_dbg_ratelimited("Too many handshakes queued, dropping packet from %pISpfsc\n", &addr);
+		if (skb_queue_len(&wg->incoming_handshakes) > MAX_QUEUED_INCOMING_HANDSHAKES) {
+			net_dbg_skb_ratelimited("Too many handshakes queued, dropping packet from %pISpfsc\n", skb);
 			goto err;
 		}
 		if (skb_linearize(skb) < 0) {
-			net_dbg_ratelimited("Unable to linearize handshake skb from %pISpfsc\n", &addr);
+			net_dbg_skb_ratelimited("Unable to linearize handshake skb from %pISpfsc\n", skb);
 			goto err;
 		}
 		skb_queue_tail(&wg->incoming_handshakes, skb);
@@ -338,7 +315,7 @@ void packet_receive(struct wireguard_device *wg, struct sk_buff *skb)
 		packet_consume_data(skb, offset, wg, receive_data_packet);
 		break;
 	default:
-		net_dbg_ratelimited("Invalid packet from %pISpfsc\n", &addr);
+		net_dbg_skb_ratelimited("Invalid packet from %pISpfsc\n", skb);
 		goto err;
 	}
 	return;
