@@ -9,8 +9,7 @@
 #include <linux/if_ether.h>
 
 struct wireguard_device;
-
-#define SKB_HEADER_LEN (max(sizeof(struct iphdr), sizeof(struct ipv6hdr)) + sizeof(struct udphdr) + NET_SKB_PAD)
+struct endpoint;
 
 int socket_init(struct wireguard_device *wg);
 void socket_uninit(struct wireguard_device *wg);
@@ -18,7 +17,7 @@ int socket_send_buffer_to_peer(struct wireguard_peer *peer, void *data, size_t l
 int socket_send_skb_to_peer(struct wireguard_peer *peer, struct sk_buff *skb, uint8_t ds);
 int socket_send_buffer_as_reply_to_skb(struct wireguard_device *wg, struct sk_buff *in_skb, void *out_buffer, size_t len);
 
-int socket_addr_from_skb(struct sockaddr_storage *sockaddr, struct sk_buff *skb);
-void socket_set_peer_addr(struct wireguard_peer *peer, struct sockaddr_storage *sockaddr);
+int socket_endpoint_from_skb(struct endpoint *endpoint, struct sk_buff *skb);
+void socket_set_peer_endpoint(struct wireguard_peer *peer, struct endpoint *endpoint);
 
 #endif
