@@ -11,10 +11,10 @@ void timers_uninit_peer_wait(struct wireguard_peer *peer);
 
 void timers_data_sent(struct wireguard_peer *peer);
 void timers_data_received(struct wireguard_peer *peer);
-void timers_any_authorized_packet_received(struct wireguard_peer *peer);
+void timers_any_authenticated_packet_received(struct wireguard_peer *peer);
 void timers_handshake_initiated(struct wireguard_peer *peer);
 void timers_handshake_complete(struct wireguard_peer *peer);
 void timers_ephemeral_key_created(struct wireguard_peer *peer);
-void timers_any_packet_sent(struct wireguard_peer *peer);
+void timers_any_authenticated_packet_traversal(struct wireguard_peer *peer);
 
 #endif
